// Sphereibl.cpp : class for rendering a solid sphere with cube mapping
// @Author: Fore June, 2014
#include <math.h>
#include <GL/glew.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include "Sphereibl.h"
#include <stdio.h>
#include <vector>
#include <string>
#include "imageio.h"
#include <iostream>

using namespace std;
   
const float Sphereibl::eyePos[] = {0, 0, 6.3, 1};
const float Sphereibl::lightPos[] = {5, 10, 5, 1};
//material same for ambient, diffuse, and specular
const float Sphereibl::materialColor[] = {1, 1, 1, 1};
const float Sphereibl::baseColor[] = {1, 0.8, 0.6};

// Create a Sphere object
Sphereibl::Sphereibl ( int  &success )
{
 string *vs, *fs;
  char *vsSource, *fsSource;

  // Read shader source code.
  readShaderFile( (char *) "sphereibl.vert",  &vsSource);
  readShaderFile( (char *) "sphereibl.frag",  &fsSource);
  vs = new string ( vsSource );
  fs = new string ( fsSource );
  success = createShader ( vs, fs );
  if ( !success ) {
    printf("infoLog:: %s\n", infoLog );
    return;
  }
  delete vs;       delete fs;
  delete vsSource; delete fsSource;

  createSphere (1.0f, N_SLICES, N_STACKS );
  int nVertices = vertices.size();
  int k = 0;
  int m = N_STACKS;
  int n = N_SLICES;

  nTriangles = 2 * n * (m - 1);    //number of triangles
  drawOrders = new short[3*nTriangles];
  for ( int j = 0; j < n; j++ )
  {
     for ( int i = 0; i < m-1; i++ ) {
       short j1 = (short)(j + 1);
       if ( j == n - 1 ) j1 = 0;   //wrap around
       short ia = (short)( j * m + i ) ;
       short ib = (short)( j * m + i + 1);
       short ic = (short) (j1 * m + i );
       short id = (short)( j1 * m + i + 1 );
       drawOrders[k++] = ia;
       drawOrders[k++] = ib;
       drawOrders[k++] = ic;
   
       drawOrders[k++] = ic;
       drawOrders[k++] = ib;
       drawOrders[k++] = id;
     }
  }

  sphereCoords = new float[3*nVertices];
  texCoords = new float[2*nVertices];
  k = 0;
  int kk = 0;
  for ( int i = 0; i < nVertices; i++ ) {
    XYZ0 v = vertices[i];
    sphereCoords[k++] = v.x;  //triangle vertex coordinates
    sphereCoords[k++] = v.y;
    sphereCoords[k++] = v.z;
  }

  init2DTexture();
  objType = 0;
}

Sphereibl::~Sphereibl()
{
  while (vertices.size() > 0 )
    vertices.pop_back();
}

GLubyte* Sphereibl::makeTexImage( char *loadfile )
{
   int i, j, c, width, height;
   GLubyte *texImage;

   /*
    *  Only works for .png or .tif images.  NULL is returned if errors occurred.
    *  loadImageRGA() is from imageio library downloaded from Internet.
    */
   texImage = loadImageRGBA( (char *) loadfile, &width, &height);
   texImageWidth = width;
   texImageHeight = height;

   return texImage;
}

void Sphereibl::init2DTexture()
{
  GLuint targets[] = {
        GL_TEXTURE_CUBE_MAP_POSITIVE_X, GL_TEXTURE_CUBE_MAP_NEGATIVE_X,
        GL_TEXTURE_CUBE_MAP_POSITIVE_Y, GL_TEXTURE_CUBE_MAP_NEGATIVE_Y,
        GL_TEXTURE_CUBE_MAP_POSITIVE_Z, GL_TEXTURE_CUBE_MAP_NEGATIVE_Z
  };
  // images rotated by 180 degrees
  const char *cubeMapImages[6] =
        {"right.png", "left.png", "top.png", "bottom.png", "front.png", "back.png"};

  glGenTextures(1, &texName);
  glActiveTexture(GL_TEXTURE0);
  glBindTexture(GL_TEXTURE_CUBE_MAP, texName);
  for ( int i = 0; i < 6; i++ ) {
    GLubyte *texImage = makeTexImage ( (char *) cubeMapImages[i] );
    glTexImage2D(targets[i], 0, GL_RGBA, texImageWidth,
                texImageHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, texImage);
    delete texImage;
  }
  glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
}
 
void Sphereibl::createSphere ( float r, int nSlices, int nStacks )
{
     double phi,  theta;
     XYZ0 *p = new XYZ0();
     XYZ0 *t = new XYZ0();   //for texture coordinates
     const double PI = 3.1415926;
     const double TWOPI = 2 * PI;

     for ( int j = 0; j < nSlices; j++ ) {
       phi = j * TWOPI / (nSlices-1);   //0 to 2pi  (modified for texture)
       for ( int i = 0; i < nStacks; i++ ) {
         theta = i * PI / (nStacks-1);  //0 to pi
         p->x = r * (float) ( sin ( theta ) *  cos ( phi ));
         p->y = r * (float) ( sin ( theta ) *  sin ( phi ));
         p->z = r * (float)  cos ( theta );
         vertices.push_back ( *p  );
       }
     }
}

void Sphereibl::draw( float mvpMatrix[4][4] ) 
{
  // get handle to vertex shader's attribute variable vPosition
  int positionHandle= glGetAttribLocation(program,"vPosition");
  // Enable a handle to the sphere vertices
  glEnableVertexAttribArray( positionHandle );
  
  // get handle to model-view projection matrix 
  int mvpMatrixHandle = glGetUniformLocation(program, "mvpMatrix");
  if (mvpMatrixHandle == -1)
     printf("No such uniform named %s\n", "mvpMatrix" );
  // pass model-view projection matrix to vertex shader
  glUniformMatrix4fv(mvpMatrixHandle, 1, GL_FALSE, &mvpMatrix[0][0] );

  int eyePosHandle = glGetUniformLocation(program, "eyePosition");
  int lightPosHandle = glGetUniformLocation(program, "lightPosition");
  int materialColorHandle = glGetUniformLocation(program, "materialColor");
  int baseColorHandle = glGetUniformLocation(program, "baseColor");
  glUniform4fv(eyePosHandle, 1, eyePos);
  glUniform4fv(lightPosHandle, 1, lightPos);
  glUniform4fv(materialColorHandle, 1, materialColor);
  glUniform3fv(baseColorHandle, 1, baseColor);
   
  
  // Prepare the sphere coordinate data
  glVertexAttribPointer(positionHandle, COORDS_PER_VERTEX,
                           GL_FLOAT, false, 0, sphereCoords);
 
  glEnable (GL_CULL_FACE);
  for ( int i = 0; i < nTriangles; i++ ) 
    glDrawElements( GL_TRIANGLES, 3,
                          GL_UNSIGNED_SHORT, (drawOrders + i*3 )); 
  
  glDisableVertexAttribArray(positionHandle);
}

